require "xrc"

module Ruboty
  module Adapters
    class Xmpp < Base
      env :RUBOTY_XMPP_USERNAME, "JID (e.g. alice@wonderland.lit)"
      env :RUBOTY_XMPP_PASSWORD, "Password"
      env :RUBOTY_XMPP_ROOMS, "Comma separated room names to join in (e.g. dev@conference.jabber.example.org)."
      env :RUBOTY_XMPP_HOST, "Optional, automatically determined from JID in absence", optional: true
      env :RUBOTY_XMPP_PORT, "Optional, default: 5222", optional: true

      def run
        init
        bind
        connect	
      end

      def say(message)
        client.say(
          body: message[:body],
          from: message[:from],
          to:   message[:to].split("/")[0],
          type: message[:original][:type],
        )
      end

      private

      def client
        @client ||= Xrc::Client.new(
          jid: jid,
          nickname: ruboty_name,
          password: password,
          room_jid: room_jids,
          hosts:    [host],
          port:     port,
        )
      end

      def init
        ruboty_name
      end

      def ruboty_name
        ENV["RUBOTY_NAME"] ||= username
      end

      def jid
        ENV["RUBOTY_XMPP_USERNAME"]
      end

      def username
        jid.split("@")[0]
      end

      def password
        ENV["RUBOTY_XMPP_PASSWORD"]
      end

      def room_jids
        ENV["RUBOTY_XMPP_ROOMS"]
      end

      def host
        ENV["RUBOTY_XMPP_HOST"] ||= jid.split("@")[1]
      end

      def port
        ENV["RUBOTY_XMPP_PORT"] ||= "5222"
      end

      def bind
        client.on_private_message(&method(:on_message))
        client.on_room_message(&method(:on_message))
      end

      def connect
        client.connect
      end

      def on_message(message)
        return if message.delayed?
        robot.receive(
          body:      message.body,
          from:      message.from,
          from_name: username_of(message),
          to:        message.to,
          type:      message.type,
        )
      end

      def username_of(message)
        case message.type
        when "groupchat"
          Xrc::Jid.new(message.from).resource
        else
          Xrc::Jid.new(message.from).node
        end
      end
    end
  end
end
